@@ -3,10 +3,10 @@
  * Copyright © 2014 Pranav Kant
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -61,6 +59,7 @@ struct _PhotosBaseItemClass
   gchar      *(*create_name_fallback)       (PhotosBaseItem *self);
   gchar      *(*create_pipeline_path)       (PhotosBaseItem *self);
   gboolean    (*create_thumbnail)           (PhotosBaseItem *self, GCancellable *cancellable, GError **error);
+  gchar      *(*create_thumbnail_path)      (PhotosBaseItem *self);
   gchar      *(*download)                   (PhotosBaseItem *self, GCancellable *cancellable, GError **error);
   GtkWidget  *(*get_source_widget)          (PhotosBaseItem *self);
   gboolean    (*metadata_add_shared)        (PhotosBaseItem  *self,
@@ -70,6 +69,7 @@ struct _PhotosBaseItemClass
                                              GCancellable    *cancellable,
                                              GError         **error);
   void        (*open)                       (PhotosBaseItem *self, GtkWindow *parent, guint32 timestamp);
+  void        (*refresh_icon)               (PhotosBaseItem *self);
   void        (*set_favorite)               (PhotosBaseItem *self, gboolean favorite);
   void        (*trash)                      (PhotosBaseItem *self);
   void        (*update_type_description)    (PhotosBaseItem *self);
@@ -89,6 +89,8 @@ cairo_surface_t    *photos_base_item_create_preview          (PhotosBaseItem *se
                                                               const gchar *first_property_name,
                                                               ...) G_GNUC_NULL_TERMINATED G_GNUC_WARN_UNUSED_RESULT;
 
+gchar              *photos_base_item_create_thumbnail_path   (PhotosBaseItem *self) G_GNUC_WARN_UNUSED_RESULT;
+
 void                photos_base_item_destroy                 (PhotosBaseItem *self);
 
 gchar              *photos_base_item_download                (PhotosBaseItem *self,
@@ -184,6 +186,8 @@ GeglNode           *photos_base_item_load_finish             (PhotosBaseItem *se
                                                               GAsyncResult *res,
                                                               GError **error);
 
+void                photos_base_item_mark_busy               (PhotosBaseItem *self);
+
 void                photos_base_item_metadata_add_shared_async  (PhotosBaseItem *self,
                                                                  const gchar *provider_type,
                                                                  const gchar *account_identity,
@@ -267,6 +271,24 @@ void                photos_base_item_pipeline_snapshot       (PhotosBaseItem *se
 
 void                photos_base_item_print                   (PhotosBaseItem *self, GtkWidget *toplevel);
 
+GFileInfo          *photos_base_item_query_info              (PhotosBaseItem *self,
+                                                              const gchar *attributes,
+                                                              GFileQueryInfoFlags flags,
+                                                              GCancellable *cancellable,
+                                                              GError **error);
+
+void                photos_base_item_query_info_async        (PhotosBaseItem *self,
+                                                              const gchar *attributes,
+                                                              GFileQueryInfoFlags flags,
+                                                              gint io_priority,
+                                                              GCancellable *cancellable,
+                                                              GAsyncReadyCallback callback,
+                                                              gpointer user_data);
+
+GFileInfo          *photos_base_item_query_info_finish       (PhotosBaseItem *self,
+                                                              GAsyncResult *res,
+                                                              GError **error);
+
 void                photos_base_item_refresh                 (PhotosBaseItem *self);
 
 void                photos_base_item_save_to_dir_async       (PhotosBaseItem *self,
@@ -311,6 +333,8 @@ void                photos_base_item_set_favorite            (PhotosBaseItem *se
 
 void                photos_base_item_trash                   (PhotosBaseItem *self);
 
+void                photos_base_item_unmark_busy             (PhotosBaseItem *self);
+
 G_END_DECLS
 
 #endif /* PHOTOS_BASE_ITEM_H */
