@@ -4,10 +4,10 @@
  * Copyright © 2012 – 2017 Red Hat, Inc.
  * Copyright © 2016 Umang Jain
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -27,6 +25,7 @@
 
 #include "config.h"
 
+#include <dazzle.h>
 #include <gexiv2/gexiv2.h>
 #include <gio/gio.h>
 #include <glib.h>
@@ -53,6 +52,39 @@ G_DEFINE_TYPE_WITH_CODE (PhotosLocalItem, photos_local_item, PHOTOS_TYPE_BASE_IT
                                                          0));
 
 
+static gboolean
+photos_local_item_source_widget_activate_link (GtkLinkButton *button, gpointer user_data)
+{
+  g_autoptr (GFile) file = NULL;
+  PhotosLocalItem *self;
+  gboolean ret_val = GDK_EVENT_PROPAGATE;
+  const gchar *uri;
+
+  g_return_val_if_fail (GTK_IS_LINK_BUTTON (button), GDK_EVENT_PROPAGATE);
+  g_return_val_if_fail (PHOTOS_IS_LOCAL_ITEM (user_data), GDK_EVENT_PROPAGATE);
+
+  self = PHOTOS_LOCAL_ITEM (user_data);
+
+  uri = photos_base_item_get_uri (PHOTOS_BASE_ITEM (self));
+  file = g_file_new_for_uri (uri);
+
+  {
+    g_autoptr (GError) error = NULL;
+
+    if (!dzl_file_manager_show (file, &error))
+      {
+        g_warning ("Unable to use org.freedesktop.FileManager1 for %s: %s", uri, error->message);
+        goto out;
+      }
+  }
+
+  ret_val = GDK_EVENT_STOP;
+
+ out:
+  return ret_val;
+}
+
+
 static gchar *
 photos_local_item_create_filename_fallback (PhotosBaseItem *item)
 {
@@ -117,6 +149,7 @@ photos_local_item_create_thumbnail (PhotosBaseItem *item, GCancellable *cancella
   const gchar *uri;
   g_autofree gchar *pipeline_path = NULL;
   g_autofree gchar *pipeline_uri = NULL;
+  g_autofree gchar *thumbnail_path = NULL;
   gint64 height;
   gint64 mtime;
   gint64 width;
@@ -128,6 +161,7 @@ photos_local_item_create_thumbnail (PhotosBaseItem *item, GCancellable *cancella
   orientation = photos_base_item_get_orientation (PHOTOS_BASE_ITEM (self));
   height = photos_base_item_get_height (PHOTOS_BASE_ITEM (self));
   width = photos_base_item_get_width (PHOTOS_BASE_ITEM (self));
+  thumbnail_path = photos_base_item_create_thumbnail_path (PHOTOS_BASE_ITEM (self));
 
   pipeline_path = photos_local_item_get_pipeline_path (self);
   pipeline_uri = photos_utils_convert_path_to_uri (pipeline_path);
@@ -139,6 +173,7 @@ photos_local_item_create_thumbnail (PhotosBaseItem *item, GCancellable *cancella
                                       height,
                                       width,
                                       pipeline_uri,
+                                      thumbnail_path,
                                       cancellable,
                                       error))
     goto out;
@@ -189,6 +224,11 @@ photos_local_item_get_source_widget (PhotosBaseItem *item)
 
       source_widget = gtk_link_button_new_with_label (source_uri, source_path);
       gtk_widget_set_halign (source_widget, GTK_ALIGN_START);
+      g_signal_connect_object (source_widget,
+                               "activate-link",
+                               G_CALLBACK (photos_local_item_source_widget_activate_link),
+                               item,
+                               0);
 
       label = gtk_bin_get_child (GTK_BIN (source_widget));
       gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
