@@ -3,10 +3,10 @@
  * Copyright © 2014 Pranav Kant
  * Copyright © 2014 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -83,6 +81,7 @@ photos_media_server_item_create_thumbnail (PhotosBaseItem *item, GCancellable *c
   g_autoptr (GFile) file = NULL;
   GQuark orientation;
   gboolean ret_val;
+  g_autofree gchar *thumbnail_path = NULL;
   const gchar *mime_type;
   const gchar *uri;
   gint64 height;
@@ -96,6 +95,7 @@ photos_media_server_item_create_thumbnail (PhotosBaseItem *item, GCancellable *c
   orientation = photos_base_item_get_orientation (item);
   height = photos_base_item_get_height (item);
   width = photos_base_item_get_width (item);
+  thumbnail_path = photos_base_item_create_thumbnail_path (item);
 
   ret_val = photos_utils_create_thumbnail (file,
                                            mime_type,
@@ -104,6 +104,7 @@ photos_media_server_item_create_thumbnail (PhotosBaseItem *item, GCancellable *c
                                            height,
                                            width,
                                            "",
+                                           thumbnail_path,
                                            cancellable,
                                            error);
 
