@@ -3,10 +3,10 @@
  * Copyright © 2012 – 2017 Red Hat, Inc.
  * Copyright © 2017 Shivam Tripathi
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -415,7 +413,7 @@ photos_properties_dialog_constructed (GObject *object)
   date_modified_str = g_date_time_format (date_modified, "%c");
 
   ctime = photos_base_item_get_date_created (item);
-  if (ctime != -1)
+  if (ctime >= 0)
     {
       g_autoptr (GDateTime) date_created = NULL;
 
