@@ -4,10 +4,10 @@
  * Copyright © 2014 Pranav Kant
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -32,7 +30,6 @@
 
 #include "photos-base-item.h"
 #include "photos-delete-notification.h"
-#include "photos-icons.h"
 #include "photos-item-manager.h"
 #include "photos-organize-collection-dialog.h"
 #include "photos-search-context.h"
@@ -47,9 +44,17 @@ struct _PhotosSelectionToolbar
   GAction *selection_mode_action;
   GHashTable *item_listeners;
   GtkWidget *toolbar_collection;
+  GtkWidget *toolbar_export;
   GtkWidget *toolbar_favorite;
+  GtkWidget *toolbar_import;
   GtkWidget *toolbar_open;
+  GtkWidget *toolbar_print;
+  GtkWidget *toolbar_properties;
+  GtkWidget *toolbar_separator;
+  GtkWidget *toolbar_share;
+  GtkWidget *toolbar_trash;
   PhotosBaseManager *item_mngr;
+  PhotosModeController *mode_cntrlr;
   PhotosSelectionController *sel_cntrlr;
   gboolean inside_refresh;
 };
@@ -194,9 +199,20 @@ photos_selection_toolbar_set_item_visibility (PhotosSelectionToolbar *self)
   GList *l;
   GList *selection;
   GtkWidget *image;
+  PhotosWindowMode mode;
   gboolean has_selection;
+  gboolean enable_collection;
+  gboolean enable_favorite;
   gboolean show_collection;
+  gboolean show_export;
   gboolean show_favorite;
+  gboolean show_import;
+  gboolean show_open;
+  gboolean show_print;
+  gboolean show_properties;
+  gboolean show_separator;
+  gboolean show_share;
+  gboolean show_trash;
   gchar *favorite_label;
   gchar *open_label;
   guint fav_count = 0;
@@ -204,11 +220,52 @@ photos_selection_toolbar_set_item_visibility (PhotosSelectionToolbar *self)
 
   self->inside_refresh = TRUE;
 
+  mode = photos_mode_controller_get_window_mode (self->mode_cntrlr);
+  switch (mode)
+    {
+    case PHOTOS_WINDOW_MODE_COLLECTION_VIEW:
+    case PHOTOS_WINDOW_MODE_COLLECTIONS:
+    case PHOTOS_WINDOW_MODE_FAVORITES:
+    case PHOTOS_WINDOW_MODE_OVERVIEW:
+    case PHOTOS_WINDOW_MODE_SEARCH:
+      show_collection = TRUE;
+      show_export = TRUE;
+      show_favorite = TRUE;
+      show_import = FALSE;
+      show_open = TRUE;
+      show_print = TRUE;
+      show_properties = TRUE;
+      show_separator = TRUE;
+      show_share = TRUE;
+      show_trash = TRUE;
+      break;
+
+    case PHOTOS_WINDOW_MODE_IMPORT:
+      show_collection = FALSE;
+      show_export = FALSE;
+      show_favorite = FALSE;
+      show_import = TRUE;
+      show_open = FALSE;
+      show_print = FALSE;
+      show_properties = FALSE;
+      show_separator = FALSE;
+      show_share = FALSE;
+      show_trash = FALSE;
+      break;
+
+    case PHOTOS_WINDOW_MODE_NONE:
+    case PHOTOS_WINDOW_MODE_EDIT:
+    case PHOTOS_WINDOW_MODE_PREVIEW:
+    default:
+      g_assert_not_reached ();
+      break;
+    }
+
   selection = photos_selection_controller_get_selection (self->sel_cntrlr);
   has_selection = selection != NULL;
 
-  show_collection = has_selection;
-  show_favorite = has_selection;
+  enable_collection = has_selection;
+  enable_favorite = has_selection;
 
   for (l = selection; l != NULL; l = g_list_next (l))
     {
@@ -218,8 +275,8 @@ photos_selection_toolbar_set_item_visibility (PhotosSelectionToolbar *self)
 
       item = PHOTOS_BASE_ITEM (photos_base_manager_get_object_by_id (self->item_mngr, urn));
 
-      show_collection = show_collection && !photos_base_item_is_collection (item);
-      show_favorite = show_favorite && !photos_base_item_is_collection (item);
+      enable_collection = enable_collection && !photos_base_item_is_collection (item);
+      enable_favorite = enable_favorite && !photos_base_item_is_collection (item);
 
       if (photos_base_item_is_favorite (item))
         fav_count++;
@@ -232,7 +289,7 @@ photos_selection_toolbar_set_item_visibility (PhotosSelectionToolbar *self)
       sel_length++;
     }
 
-  show_favorite = show_favorite && ((fav_count == 0) || (fav_count == sel_length));
+  enable_favorite = enable_favorite && ((fav_count == 0) || (fav_count == sel_length));
 
   if (apps != NULL && apps->next == NULL) /* length == 1 */
     /* Translators: this is the Open action in a context menu */
@@ -245,23 +302,34 @@ photos_selection_toolbar_set_item_visibility (PhotosSelectionToolbar *self)
   g_free (open_label);
   g_list_free_full (apps, g_free);
 
-  if (show_favorite && fav_count == sel_length)
+  if (enable_favorite && fav_count == sel_length)
     {
       favorite_label = g_strdup (_("Remove from favorites"));
-      image = gtk_image_new_from_icon_name (PHOTOS_ICON_FAVORITE_SYMBOLIC, GTK_ICON_SIZE_BUTTON);
+      image = gtk_image_new_from_icon_name ("starred-symbolic", GTK_ICON_SIZE_BUTTON);
     }
   else
     {
       favorite_label = g_strdup (_("Add to favorites"));
-      image = gtk_image_new_from_icon_name (PHOTOS_ICON_NOT_FAVORITE_SYMBOLIC, GTK_ICON_SIZE_BUTTON);
+      image = gtk_image_new_from_icon_name ("non-starred-symbolic", GTK_ICON_SIZE_BUTTON);
     }
 
   gtk_button_set_image (GTK_BUTTON (self->toolbar_favorite), image);
   gtk_widget_set_tooltip_text (self->toolbar_favorite, favorite_label);
   g_free (favorite_label);
 
-  gtk_widget_set_sensitive (self->toolbar_collection, show_collection);
-  gtk_widget_set_sensitive (self->toolbar_favorite, show_favorite);
+  gtk_widget_set_sensitive (self->toolbar_collection, enable_collection);
+  gtk_widget_set_sensitive (self->toolbar_favorite, enable_favorite);
+
+  gtk_widget_set_visible (self->toolbar_collection, show_collection);
+  gtk_widget_set_visible (self->toolbar_export, show_export);
+  gtk_widget_set_visible (self->toolbar_favorite, show_favorite);
+  gtk_widget_set_visible (self->toolbar_import, show_import);
+  gtk_widget_set_visible (self->toolbar_open, show_open);
+  gtk_widget_set_visible (self->toolbar_print, show_print);
+  gtk_widget_set_visible (self->toolbar_properties, show_properties);
+  gtk_widget_set_visible (self->toolbar_separator, show_separator);
+  gtk_widget_set_visible (self->toolbar_share, show_share);
+  gtk_widget_set_visible (self->toolbar_trash, show_trash);
 
   self->inside_refresh = FALSE;
 }
@@ -324,6 +392,7 @@ photos_selection_toolbar_dispose (GObject *object)
 
   g_clear_pointer (&self->item_listeners, (GDestroyNotify) g_hash_table_unref);
   g_clear_object (&self->item_mngr);
+  g_clear_object (&self->mode_cntrlr);
   g_clear_object (&self->sel_cntrlr);
 
   G_OBJECT_CLASS (photos_selection_toolbar_parent_class)->dispose (object);
@@ -352,6 +421,7 @@ photos_selection_toolbar_init (PhotosSelectionToolbar *self)
   gtk_widget_init_template (GTK_WIDGET (self));
 
   self->item_mngr = g_object_ref (state->item_mngr);
+  self->mode_cntrlr = g_object_ref (state->mode_cntrlr);
 
   self->sel_cntrlr = photos_selection_controller_dup_singleton ();
   g_signal_connect_object (self->sel_cntrlr,
@@ -378,8 +448,15 @@ photos_selection_toolbar_class_init (PhotosSelectionToolbarClass *class)
   object_class->dispose = photos_selection_toolbar_dispose;
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Photos/selection-toolbar.ui");
+  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_export);
   gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_favorite);
+  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_import);
   gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_open);
+  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_print);
+  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_properties);
+  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_separator);
+  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_share);
+  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_trash);
   gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_collection);
   gtk_widget_class_bind_template_callback (widget_class, photos_selection_toolbar_favorite_clicked);
   gtk_widget_class_bind_template_callback (widget_class, photos_selection_toolbar_collection_clicked);
