@@ -3,10 +3,10 @@
  * Copyright © 2012 – 2017 Red Hat, Inc.
  * Copyright © 2009 Yorba Foundation
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -28,13 +26,13 @@
 
 #include "config.h"
 
-#include <string.h>
-
+#include <gdk/gdk.h>
 #include <glib.h>
 #include <tracker-sparql.h>
 #include <libgd/gd.h>
 
 #include "photos-application.h"
+#include "photos-device-item.h"
 #include "photos-enums.h"
 #include "photos-error.h"
 #include "photos-facebook-item.h"
@@ -46,6 +44,7 @@
 #include "photos-offset-collection-view-controller.h"
 #include "photos-offset-collections-controller.h"
 #include "photos-offset-favorites-controller.h"
+#include "photos-offset-import-controller.h"
 #include "photos-offset-overview-controller.h"
 #include "photos-offset-search-controller.h"
 #include "photos-query.h"
@@ -63,27 +62,15 @@
 #include "photos-tracker-collection-view-controller.h"
 #include "photos-tracker-collections-controller.h"
 #include "photos-tracker-favorites-controller.h"
+#include "photos-tracker-import-controller.h"
 #include "photos-tracker-overview-controller.h"
 #include "photos-tracker-queue.h"
 #include "photos-tracker-search-controller.h"
 #include "photos-utils.h"
 
 
-typedef struct _PhotosUtilsFileQueryInfoData PhotosUtilsFileQueryInfoData;
-
-struct _PhotosUtilsFileQueryInfoData
-{
-  GFileQueryInfoFlags flags;
-  gchar *attributes;
-};
-
 static const gdouble EPSILON = 1e-5;
 
-enum
-{
-  THUMBNAIL_GENERATION = 0
-};
-
 
 GdkPixbuf *
 photos_utils_center_pixbuf (GdkPixbuf *pixbuf, gint size)
@@ -360,6 +347,7 @@ photos_utils_create_thumbnail (GFile *file,
                                gint64 original_height,
                                gint64 original_width,
                                const gchar *pipeline_uri,
+                               const gchar *thumbnail_path,
                                GCancellable *cancellable,
                                GError **error)
 {
@@ -374,6 +362,7 @@ photos_utils_create_thumbnail (GFile *file,
                                                     original_height,
                                                     original_width,
                                                     pipeline_uri,
+                                                    thumbnail_path,
                                                     cancellable,
                                                     error))
     goto out;
@@ -420,24 +409,26 @@ photos_utils_create_zoom_target_value (gdouble delta, PhotosZoomEvent event)
 
 
 static GIcon *
-photos_utils_get_thumbnail_icon (const gchar *uri)
+photos_utils_get_thumbnail_icon (PhotosBaseItem *item)
 {
-  g_autoptr (GFile) file = NULL;
   g_autoptr (GFile) thumb_file = NULL;
   g_autoptr (GFileInfo) info = NULL;
   GIcon *icon = NULL;
   const gchar *thumb_path;
+  const gchar *uri;
 
-  file = g_file_new_for_uri (uri);
+  uri = photos_base_item_get_uri (item);
+  if (uri == NULL || uri[0] == '\0')
+    goto out;
 
   {
     g_autoptr (GError) error = NULL;
 
-    info = photos_utils_file_query_info (file,
-                                         G_FILE_ATTRIBUTE_THUMBNAIL_PATH,
-                                         G_FILE_QUERY_INFO_NONE,
-                                         NULL,
-                                         &error);
+    info = photos_base_item_query_info (item,
+                                        G_FILE_ATTRIBUTE_THUMBNAIL_PATH,
+                                        G_FILE_QUERY_INFO_NONE,
+                                        NULL,
+                                        &error);
     if (error != NULL)
       {
         g_warning ("Unable to fetch thumbnail path for %s: %s", uri, error->message);
@@ -458,15 +449,14 @@ photos_utils_get_thumbnail_icon (const gchar *uri)
 
 
 GIcon *
-photos_utils_get_icon_from_cursor (TrackerSparqlCursor *cursor)
+photos_utils_get_icon_from_item (PhotosBaseItem *item)
 {
   GIcon *icon = NULL;
   gboolean is_remote = FALSE;
   const gchar *identifier;
   const gchar *mime_type;
-  const gchar *rdf_type;
 
-  identifier = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_IDENTIFIER, NULL);
+  identifier = photos_base_item_get_identifier (item);
   if (identifier != NULL)
     {
       if (g_str_has_prefix (identifier, "facebook:") ||
@@ -476,27 +466,25 @@ photos_utils_get_icon_from_cursor (TrackerSparqlCursor *cursor)
     }
 
   if (!is_remote)
-    {
-      const gchar *uri;
-
-      uri = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_URI, NULL);
-      if (uri != NULL)
-        icon = photos_utils_get_thumbnail_icon (uri);
-    }
+    icon = photos_utils_get_thumbnail_icon (item);
 
   if (icon != NULL)
     goto out;
 
-  mime_type = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_MIME_TYPE, NULL);
+  mime_type = photos_base_item_get_mime_type (item);
   if (mime_type != NULL)
     icon = g_content_type_get_icon (mime_type);
 
   if (icon != NULL)
     goto out;
 
-  rdf_type = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_RDF_TYPE, NULL);
-  if (mime_type != NULL)
-    icon = photos_utils_icon_from_rdf_type (rdf_type);
+  if (photos_base_item_is_collection (item))
+    {
+      gint size;
+
+      size = photos_utils_get_icon_size ();
+      icon = photos_utils_create_collection_icon (size, NULL);
+    }
 
   if (icon != NULL)
     goto out;
@@ -705,6 +693,7 @@ photos_utils_ensure_builtins (void)
 
   if (g_once_init_enter (&once_init_value))
     {
+      g_type_ensure (PHOTOS_TYPE_DEVICE_ITEM);
       g_type_ensure (PHOTOS_TYPE_FACEBOOK_ITEM);
       g_type_ensure (PHOTOS_TYPE_FLICKR_ITEM);
       g_type_ensure (PHOTOS_TYPE_GOOGLE_ITEM);
@@ -722,6 +711,7 @@ photos_utils_ensure_builtins (void)
       g_type_ensure (PHOTOS_TYPE_TRACKER_COLLECTION_VIEW_CONTROLLER);
       g_type_ensure (PHOTOS_TYPE_TRACKER_COLLECTIONS_CONTROLLER);
       g_type_ensure (PHOTOS_TYPE_TRACKER_FAVORITES_CONTROLLER);
+      g_type_ensure (PHOTOS_TYPE_TRACKER_IMPORT_CONTROLLER);
       g_type_ensure (PHOTOS_TYPE_TRACKER_OVERVIEW_CONTROLLER);
       g_type_ensure (PHOTOS_TYPE_TRACKER_SEARCH_CONTROLLER);
 
@@ -850,145 +840,6 @@ photos_utils_file_copy_as_thumbnail (GFile *source,
 }
 
 
-GFileInfo *
-photos_utils_file_query_info (GFile *file,
-                              const gchar *attributes,
-                              GFileQueryInfoFlags flags,
-                              GCancellable *cancellable,
-                              GError **error)
-{
-  GFileAttributeMatcher *matcher = NULL; /* TODO: use g_autoptr */
-  g_autoptr (GFileInfo) info = NULL;
-  GFileInfo *ret_val = NULL;
-
-  g_return_val_if_fail (G_IS_FILE (file), NULL);
-  g_return_val_if_fail (attributes != NULL && attributes[0] != '\0', NULL);
-  g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
-  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-
-  info = g_file_query_info (file, attributes, flags, cancellable, error);
-  if (info == NULL)
-    goto out;
-
-  matcher = g_file_attribute_matcher_new (attributes);
-  if (g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_THUMBNAIL_IS_VALID)
-      || g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_THUMBNAIL_PATH)
-      || g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_THUMBNAILING_FAILED))
-    {
-      g_autofree gchar *path = NULL;
-
-      g_file_info_remove_attribute (info, G_FILE_ATTRIBUTE_THUMBNAIL_IS_VALID);
-      g_file_info_remove_attribute (info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH);
-      g_file_info_remove_attribute (info, G_FILE_ATTRIBUTE_THUMBNAILING_FAILED);
-
-      path = photos_utils_get_thumbnail_path_for_file (file);
-      if (g_file_test (path, G_FILE_TEST_IS_REGULAR))
-        {
-          g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_THUMBNAIL_IS_VALID, TRUE);
-          g_file_info_set_attribute_byte_string (info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH, path);
-          g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_THUMBNAILING_FAILED, FALSE);
-        }
-    }
-
-  ret_val = g_object_ref (info);
-
- out:
-  g_clear_pointer (&matcher, (GDestroyNotify) g_file_attribute_matcher_unref);
-  return ret_val;
-}
-
-
-static void
-photos_utils_file_query_info_data_free (PhotosUtilsFileQueryInfoData *data)
-{
-  g_free (data->attributes);
-  g_slice_free (PhotosUtilsFileQueryInfoData, data);
-}
-
-
-static PhotosUtilsFileQueryInfoData *
-photos_utils_file_query_info_data_new (const gchar *attributes, GFileQueryInfoFlags flags)
-{
-  PhotosUtilsFileQueryInfoData *data;
-
-  data = g_slice_new0 (PhotosUtilsFileQueryInfoData);
-  data->flags = flags;
-  data->attributes = g_strdup (attributes);
-
-  return data;
-}
-
-
-static void
-photos_utils_file_query_info_in_thread_func (GTask *task,
-                                             gpointer source_object,
-                                             gpointer task_data,
-                                             GCancellable *cancellable)
-{
-  GError *error;
-  GFile *file = G_FILE (source_object);
-  g_autoptr (GFileInfo) info = NULL;
-  PhotosUtilsFileQueryInfoData *data = (PhotosUtilsFileQueryInfoData *) task_data;
-
-  error = NULL;
-  info = photos_utils_file_query_info (file, data->attributes, data->flags, cancellable, &error);
-  if (error != NULL)
-    {
-      g_task_return_error (task, error);
-      goto out;
-    }
-
-  g_task_return_pointer (task, g_object_ref (info), g_object_unref);
-
- out:
-  return;
-}
-
-
-void
-photos_utils_file_query_info_async (GFile *file,
-                                    const gchar *attributes,
-                                    GFileQueryInfoFlags flags,
-                                    gint io_priority,
-                                    GCancellable *cancellable,
-                                    GAsyncReadyCallback callback,
-                                    gpointer user_data)
-{
-  g_autoptr (GTask) task = NULL;
-  PhotosUtilsFileQueryInfoData *data;
-  const gchar *wildcard;
-
-  g_return_if_fail (G_IS_FILE (file));
-  g_return_if_fail (attributes != NULL && attributes[0] != '\0');
-  g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
-
-  wildcard = strstr (attributes, "*");
-  g_return_if_fail (wildcard == NULL);
-
-  data = photos_utils_file_query_info_data_new (attributes, flags);
-
-  task = g_task_new (file, cancellable, callback, user_data);
-  g_task_set_priority (task, io_priority);
-  g_task_set_source_tag (task, photos_utils_file_query_info_async);
-  g_task_set_task_data (task, data, (GDestroyNotify) photos_utils_file_query_info_data_free);
-
-  g_task_run_in_thread (task, photos_utils_file_query_info_in_thread_func);
-}
-
-
-GFileInfo *
-photos_utils_file_query_info_finish (GFile *file, GAsyncResult *res, GError **error)
-{
-  GTask *task = G_TASK (res);
-
-  g_return_val_if_fail (g_task_is_valid (res, file), NULL);
-  g_return_val_if_fail (g_task_get_source_tag (task) == photos_utils_file_query_info_async, NULL);
-  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-
-  return g_task_propagate_pointer (task, error);
-}
-
-
 void
 photos_utils_get_controller (PhotosWindowMode mode,
                              PhotosOffsetController **out_offset_cntrlr,
@@ -1018,6 +869,11 @@ photos_utils_get_controller (PhotosWindowMode mode,
       trk_cntrlr = photos_tracker_favorites_controller_dup_singleton ();
       break;
 
+    case PHOTOS_WINDOW_MODE_IMPORT:
+      offset_cntrlr = photos_offset_import_controller_dup_singleton ();
+      trk_cntrlr = photos_tracker_import_controller_dup_singleton ();
+      break;
+
     case PHOTOS_WINDOW_MODE_OVERVIEW:
       offset_cntrlr = photos_offset_overview_controller_dup_singleton ();
       trk_cntrlr = photos_tracker_overview_controller_dup_singleton ();
@@ -1164,47 +1020,6 @@ photos_utils_get_selection_mode (void)
 }
 
 
-gchar *
-photos_utils_get_thumbnail_path_for_file (GFile *file)
-{
-  gchar *path;
-  g_autofree gchar *uri = NULL;
-
-  uri = g_file_get_uri (file);
-  path = photos_utils_get_thumbnail_path_for_uri (uri);
-
-  return path;
-}
-
-
-gchar *
-photos_utils_get_thumbnail_path_for_uri (const gchar *uri)
-{
-  const gchar *cache_dir;
-  g_autofree gchar *filename = NULL;
-  g_autofree gchar *md5 = NULL;
-  gchar *path;
-  g_autofree gchar *thumbnails_subdir = NULL;
-  gint size;
-
-  md5 = g_compute_checksum_for_string (G_CHECKSUM_MD5, uri, -1);
-  filename = g_strconcat (md5, ".png", NULL);
-
-  cache_dir = g_get_user_cache_dir ();
-  size = photos_utils_get_icon_size ();
-  thumbnails_subdir = g_strdup_printf ("%d-%d", size, THUMBNAIL_GENERATION);
-
-  path = g_build_filename (cache_dir,
-                           PACKAGE_TARNAME,
-                           "thumbnails",
-                           thumbnails_subdir,
-                           filename,
-                           NULL);
-
-  return path;
-}
-
-
 GList *
 photos_utils_get_urns_from_items (GList *items)
 {
@@ -1239,20 +1054,6 @@ photos_utils_get_version (void)
 }
 
 
-GIcon *
-photos_utils_icon_from_rdf_type (const gchar *type)
-{
-  GIcon *ret_val = NULL;
-  gint size;
-
-  size = photos_utils_get_icon_size ();
-  if (strstr (type, "nfo#DataContainer") != NULL)
-    ret_val = photos_utils_create_collection_icon (size, NULL);
-
-  return ret_val;
-}
-
-
 void
 photos_utils_list_box_header_func (GtkListBoxRow *row, GtkListBoxRow *before, gpointer user_data)
 {
@@ -1274,6 +1075,32 @@ photos_utils_list_box_header_func (GtkListBoxRow *row, GtkListBoxRow *before, gp
 }
 
 
+GAppLaunchContext *
+photos_utils_new_app_launch_context_from_widget (GtkWidget *widget)
+{
+  GAppLaunchContext *ret_val = NULL;
+  g_autoptr (GdkAppLaunchContext) ctx = NULL;
+  GdkDisplay *display = NULL;
+  GdkScreen *screen = NULL;
+
+  if (widget != NULL)
+    {
+      screen = gtk_widget_get_screen (widget);
+      display = gdk_screen_get_display (screen);
+    }
+
+  if (display == NULL)
+    display = gdk_display_get_default ();
+
+  ctx = gdk_display_get_app_launch_context (display);
+  if (screen != NULL)
+    gdk_app_launch_context_set_screen (ctx, screen);
+
+  ret_val = G_APP_LAUNCH_CONTEXT (g_steal_pointer (&ctx));
+  return ret_val;
+}
+
+
 void
 photos_utils_object_list_free_full (GList *objects)
 {
