@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -32,7 +30,6 @@
 #include <gio/gio.h>
 #include <glib.h>
 #include <gtk/gtk.h>
-#include <tracker-sparql.h>
 
 #include "photos-base-item.h"
 #include "photos-base-manager.h"
@@ -51,6 +48,7 @@ G_BEGIN_DECLS
 #define PHOTOS_COLLECTION_SCREENSHOT \
   "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#image-category-screenshot"
 #define PHOTOS_EXPORT_SUBPATH "Exports"
+#define PHOTOS_IMPORT_SUBPATH "Imports"
 
 typedef enum
 {
@@ -78,12 +76,13 @@ gboolean         photos_utils_create_thumbnail            (GFile *file,
                                                            gint64 original_height,
                                                            gint64 original_width,
                                                            const gchar *pipeline_uri,
+                                                           const gchar *thumbnail_path,
                                                            GCancellable *cancellable,
                                                            GError **error);
 
 GVariant        *photos_utils_create_zoom_target_value    (gdouble delta, PhotosZoomEvent event);
 
-GIcon           *photos_utils_get_icon_from_cursor        (TrackerSparqlCursor *cursor);
+GIcon           *photos_utils_get_icon_from_item          (PhotosBaseItem *item);
 
 gdouble          photos_utils_get_zoom_delta              (GVariant *dictionary);
 
@@ -125,22 +124,6 @@ gboolean         photos_utils_file_copy_as_thumbnail      (GFile *source,
                                                            GCancellable *cancellable,
                                                            GError **error);
 
-GFileInfo       *photos_utils_file_query_info             (GFile *file,
-                                                           const gchar *attributes,
-                                                           GFileQueryInfoFlags flags,
-                                                           GCancellable *cancellable,
-                                                           GError **error);
-
-void             photos_utils_file_query_info_async       (GFile *file,
-                                                           const gchar *attributes,
-                                                           GFileQueryInfoFlags flags,
-                                                           gint io_priority,
-                                                           GCancellable *cancellable,
-                                                           GAsyncReadyCallback callback,
-                                                           gpointer user_data);
-
-GFileInfo       *photos_utils_file_query_info_finish      (GFile *file, GAsyncResult *res, GError **error);
-
 void             photos_utils_get_controller              (PhotosWindowMode mode,
                                                            PhotosOffsetController **out_offset_cntrlr,
                                                            PhotosTrackerController **out_trk_cntrlr);
@@ -158,20 +141,16 @@ const gchar     *photos_utils_get_provider_name           (PhotosBaseManager *sr
 
 gboolean         photos_utils_get_selection_mode          (void);
 
-gchar           *photos_utils_get_thumbnail_path_for_file (GFile *file);
-
-gchar           *photos_utils_get_thumbnail_path_for_uri  (const gchar *uri);
-
 GList           *photos_utils_get_urns_from_items         (GList *items);
 
 const gchar     *photos_utils_get_version                 (void);
 
-GIcon           *photos_utils_icon_from_rdf_type          (const gchar *type);
-
 void             photos_utils_list_box_header_func        (GtkListBoxRow *row,
                                                            GtkListBoxRow *before,
                                                            gpointer user_data);
 
+GAppLaunchContext *photos_utils_new_app_launch_context_from_widget (GtkWidget *widget);
+
 void             photos_utils_object_list_free_full       (GList *objects);
 
 gchar           *photos_utils_print_zoom_action_detailed_name (const gchar *action_name,
